@@ -9,9 +9,11 @@ inside the test directory:
 """
 
 import io
-import subprocess
+import os
+import pathlib
 import textwrap
 from lxml import etree
+from tempfile import NamedTemporaryFile
 
 from reportlab.graphics.shapes import (
     _CLOSEPATH, _CURVETO, _LINETO, _MOVETO, Group, Path, Polygon, PolyLine, Rect,
@@ -30,18 +32,67 @@ def _testit(func, mapping):
 
     failed = []
     for input, expected in mapping:
-        result = func(input)
+        if isinstance(input, dict):
+            result = func(**input)
+        else:
+            result = func(input)
         if not result == expected:
             failed.append((input, result, expected))
 
     if failed:
         print("failed tests (input, result, expected):")
         for input, result, expected in failed:
-            print("  %s : %s != %s" % (repr(input), result, expected))
+            print(f"  {input!r} : {result} != {expected}")
 
     return failed
 
 
+class TestSvg2rlgInput:
+    test_content = textwrap.dedent('''\
+        <?xml version="1.0"?>
+        <svg xmlns="http://www.w3.org/2000/svg"
+             width="1200" height="800"
+             viewBox="0 0 36 24">
+            <rect y="10" width="36" height="4"/>
+        </svg>
+    ''')
+
+    def test_path_as_string_input(self):
+        try:
+            with NamedTemporaryFile(mode='w', suffix='.svg', delete=False) as fp:
+                fp.write(self.test_content)
+                file_path = fp.name
+            drawing = svglib.svg2rlg(file_path)
+            assert drawing is not None
+        finally:
+            os.unlink(file_path)
+
+    def test_path_as_pathlib_input(self):
+        try:
+            with NamedTemporaryFile(mode='w', suffix='.svg', delete=False) as fp:
+                fp.write(self.test_content)
+                file_path = pathlib.Path(fp.name)
+            drawing = svglib.svg2rlg(file_path)
+            assert drawing is not None
+        finally:
+            os.unlink(file_path)
+
+    def test_filelike_input(self):
+        drawing = svglib.svg2rlg(io.StringIO(self.test_content))
+        assert drawing is not None
+
+    def test_filehandle_input(self):
+        try:
+            with NamedTemporaryFile(mode='w', suffix='.svg', delete=False) as fp:
+                fp.write(self.test_content)
+                file_path = fp.name
+            with open(file_path, 'rb') as fp:
+                drawing = svglib.svg2rlg(fp)
+                assert drawing is not None
+        finally:
+            os.unlink(file_path)
+
+
 class TestPaths:
     """Testing path-related code."""
 
@@ -104,6 +155,12 @@ class TestPaths:
                  'c', [-1.438, -0.502, -1.684, -0.494, -2.405, 0.058],
                  'c', [-1.618, 1.239, -3.869, 1.355, -5.894, 0.299],
                  'z', []]),
+
+            # scientific notation
+            ("M 114.2,1.919e2 1e-4,22.5E-1",
+                ["M", [114.2, 191.9], 'L', [0.0001, 2.25]]),
+            ("a1518e-3 .1518E2  0 011.195E+2-.547",
+                ["a", [1.518, 15.18, 0, 0, 1, 119.5, -0.547]]),
         )
         failed = _testit(utils.normalise_svg_path, mapping)
         assert len(failed) == 0
@@ -166,13 +223,14 @@ class TestPaths:
         assert group is None
 
     def test_clipped_path(self):
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg version="1.1" xmlns="http://www.w3.org/2000/svg"
                  xmlns:xlink="http://www.w3.org/1999/xlink" width="660" height="480">
                 <defs>
                     <path id="my-clipping-path" d="M99,176 L 110 170 112 172Z"/>
-                    <rect id="my-clipping-rect" x="88.155" y="163" width="419.69" height="20.68"/>
+                    <rect id="my-clipping-rect" x="88.155" y="163" width="419.69" height="20.68"
+                          fill="red" stroke="blue"/>
                 </defs>
                 <clipPath id="my-clip-path">
                     <use xlink:href="#my-clipping-path"/>
@@ -186,7 +244,11 @@ class TestPaths:
         ''')))
         assert isinstance(drawing.contents[0].contents[0].contents[0], svglib.ClippingPath)
         # Clipping rect was converted to a path
-        assert isinstance(drawing.contents[0].contents[1].contents[0], svglib.ClippingPath)
+        rect_clip = drawing.contents[0].contents[1].contents[0]
+        assert isinstance(rect_clip, svglib.ClippingPath)
+        # Fill and stroke properties are force-deleted.
+        assert rect_clip.getProperties()['fillColor'] is None
+        assert rect_clip.getProperties()['strokeColor'] is None
 
 
 def force_cmyk(rgb):
@@ -203,12 +265,12 @@ class TestColorAttrConverter:
         mapping = (
             ("red", colors.red),
             ("#ff0000", colors.red),
-            ("#ff000055", colors.Color(1, 0, 0, 1/3.0)),
+            ("#ff000055", colors.Color(1, 0, 0, 1 / 3)),
             ("#f00", colors.red),
             ("#f00f", colors.red),
-            ("rgb(100%,50%,10%)", colors.Color(1, 1.0/255 * 128, 1.0/255 * 26, 1)),
+            ("rgb(100%,50%,10%)", colors.Color(1, 128 / 255, 26 / 255, 1)),
             ("rgb(255, 0, 0)", colors.red),
-            ("rgba(255, 255, 128, .5)", colors.Color(1, 1, 1.0/255 * 128, .5)),
+            ("rgba(255, 255, 128, .5)", colors.Color(1, 1, 128 / 255, .5)),
             ("fuchsia", colors.Color(1, 0, 1, 1)),
             ("slategrey", colors.HexColor(0x708090)),
             ("transparent", colors.Color(0, 0, 0, 0)),
@@ -238,7 +300,7 @@ class TestColorAttrConverter:
 class TestLengthAttrConverter:
     "Testing length attribute conversion."
 
-    def test_0(self):
+    def test_length_conversion(self):
         "Test length attribute conversion."
 
         mapping = (
@@ -274,11 +336,7 @@ class TestLengthAttrConverter:
         length = ac.convertLength("1e1%", attr_name='height')
         assert length == 15
 
-
-class TestLengthListAttrConverter:
-    "Testing length attribute conversion."
-
-    def test_0(self):
+    def test_length_list_conversion(self):
         "Test length list attribute conversion."
 
         mapping = (
@@ -353,7 +411,7 @@ class TestAttrConverter:
         """
         Any shape with no fill property should set black color in rlg syntax.
         """
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg xmlns="http://www.w3.org/2000/svg"
                  width="1200" height="800"
@@ -367,7 +425,7 @@ class TestAttrConverter:
         """
         The fill-opacity property set the alpha of the color.
         """
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg version="1.1"
                  xmlns="http://www.w3.org/2000/svg"
@@ -378,6 +436,22 @@ class TestAttrConverter:
         ''')))
         assert drawing.contents[0].contents[0].fillColor == colors.Color(0, 0, 0, 0)
 
+    def test_fillcolor_alpha_set_fillopacity(self):
+        converter = svglib.Svg2RlgShapeConverter(None)
+        node = etree.XML('<polygon fill="#ff000080"/>')
+        poly = Polygon()
+        converter.applyStyleOnShape(poly, node)
+        assert poly.fillOpacity == 128 / 255
+        assert poly.strokeOpacity == 1
+
+    def test_strokecolor_alpha_set_strokeopacity(self):
+        converter = svglib.Svg2RlgShapeConverter(None)
+        node = etree.XML('<polygon stroke="#cccccca0"/>')
+        poly = Polygon()
+        converter.applyStyleOnShape(poly, node)
+        assert poly.fillOpacity == 1
+        assert poly.strokeOpacity == 160 / 255
+
     def test_fillrule(self):
         converter = svglib.Svg2RlgShapeConverter(None)
         node = etree.XML('<polygon fill-rule="evenodd"/>')
@@ -406,15 +480,37 @@ class TestApplyTransformOnGroup:
         converter.applyTransformOnGroup(transform, group)
         assert group.transform == (1, 0, 0, 1, 10, 0)
 
+    def test_transform_matrix(self):
+        NEUTRAL_TRANSFORM = (1, 0, 0, 1, 0, 0)
+        group = Group()
+        converter = svglib.Svg2RlgShapeConverter(None)
+        # if the values are not 6, ignore
+        transform = "matrix(1 2 3 4 5)"
+        converter.applyTransformOnGroup(transform, group)
+        assert group.transform == NEUTRAL_TRANSFORM
+
+        # Matrix-only
+        group = Group()
+        transform = "matrix(3 1 -1 3 30 40)"
+        converter.applyTransformOnGroup(transform, group)
+        assert group.transform == (3, 1, -1, 3, 30, 40)
+
+        # Combination of matrix and translation
+        group = Group()
+        transform = "translate(40 40) matrix(3 1 -1 3 30 40)"
+        converter.applyTransformOnGroup(transform, group)
+        assert group.transform == (3, 1, -1, 3, 70, 80)
+
 
 class TestStyleSheets:
     def test_css_stylesheet(self):
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg width="777" height="267" xml:space="preserve">
               <defs>
                 <style type="text/css">
-                #p1 { fill:rgb(255,0,0); }
+                path { fill:none; }
+                #p1 { fill:rgb(255,0,0) !important; }
                 #p2 { fill:rgb(255,0,0); }
                 .paths { stroke-width:1.5; }
                 </style>
@@ -432,35 +528,82 @@ class TestStyleSheets:
         assert main_group.contents[0].contents[1].contents[0].strokeWidth == 1.5
 
 
+class TestGroupNode:
+    def test_svg_groups_have_svgid(self):
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
+            <?xml version="1.0"?>
+            <svg width="777" height="267">
+                <g id="g856">
+                    <rect
+                         y="107.07929"
+                         x="64.942139"
+                         height="19.506001"
+                         width="34.690311"
+                         id="rect850"
+                </g>
+            </svg>
+        ''')))
+        main_group = drawing.contents[0]
+
+        gr, = main_group.contents
+        assert gr.svgid == 'g856'
+        assert isinstance(gr, Group)
+
+    def test_created_groups_have_svgid_of_their_content(self):
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
+            <?xml version="1.0"?>
+            <svg width="777" height="267">
+                <g id="g856">
+                    <rect
+                         y="107.07929"
+                         x="64.942139"
+                         height="19.506001"
+                         width="34.690311"
+                         id="rect850"
+                    <rect
+                         y="136.5135"
+                         x="108.62624"
+                         height="17.637161"
+                         width="29.083796"
+                         id="rect852"
+                    <path
+                       d="M 601.06712,388.63166 H 954.67961"
+                       id="path819" />
+                </g>
+            </svg>
+        ''')))
+        main_group = drawing.contents[0]
+        gr, = main_group.contents
+
+        r1_gr, r2_gr, pth_gr = gr.contents
+        assert r1_gr.svgid == 'rect850'
+        assert r2_gr.svgid == 'rect852'
+        assert pth_gr.svgid == 'path819'
+        assert isinstance(pth_gr, Group)
+
+    def test_svg_layers_have_label(self):
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
+            <?xml version="1.0"?>
+            <svg width="777" height="267">
+                <g inkscape:groupmode="layer"
+                 id="layer2"
+                 inkscape:label="x_axis"
+                 style="display:inline"
+                 transform="translate(-476.20282,35.510971)">
+                <path
+                   d="M 601.06712,388.63166 H 954.67961"
+                   id="path819" />
+                </g>
+            </svg>
+        ''')))
+        main_group = drawing.contents[0]
+        gr, = main_group.contents
+        assert gr.label == "x_axis"
+
+
 class TestTextNode:
-    def test_font_family(self):
-        def font_config_available():
-            try:
-                subprocess.call(["fc-match"])
-            except OSError:
-                return False
-            return True
-
-        converter = svglib.Svg2RlgAttributeConverter()
-        # Check PDF standard names are untouched
-        assert converter.convertFontFamily('ZapfDingbats') == 'ZapfDingbats'
-        assert converter.convertFontFamily('bilbo ZapfDingbats') == 'ZapfDingbats'
-        assert converter.convertFontFamily(' bilbo    ZapfDingbats  ') == 'ZapfDingbats'
-        assert converter.convertFontFamily(' bilbo,    ZapfDingbats  ') == 'ZapfDingbats'
-        if font_config_available():
-            # Fontconfig will always provide at least a default font and register
-            # that font under the provided font name.
-            assert converter.convertFontFamily('SomeFont') == 'SomeFont'
-        else:
-            # Unknown fonts are converted to Helvetica by default.
-            assert converter.convertFontFamily('SomeFont') == 'Helvetica'
-        # Check font names with spaces
-        assert converter.split_attr_list("'Open Sans', Arial, 'New Times Roman'") == [
-            'Open Sans', 'Arial', 'New Times Roman'
-        ]
-
     def test_space_preservation(self):
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg width="777" height="267">
               <text style="fill:#000000; stroke:none; font-size:28;">
@@ -475,7 +618,7 @@ class TestTextNode:
         assert len(main_group.contents[0].contents) == 2
         assert main_group.contents[0].contents[0].text == "TITLE 1"
 
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg width="777" height="267" xml:space="preserve">
               <text style="fill:#000000; stroke:none; font-size:28;">
@@ -488,7 +631,7 @@ class TestTextNode:
         assert main_group.contents[0].contents[0].text == '     '
         assert main_group.contents[0].contents[1].text == "TITLE    1"
 
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg width="777" height="267">
               <text style="fill:#000000; stroke:none; font-size:28;">
@@ -508,7 +651,7 @@ class TestTextNode:
         The x/y positions of a tspan are either relative to the current text
         position, or can be absoluted by specifying the x/y attributes.
         """
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg width="777" height="267">
               <text x="10" y="20" style="fill:#000000; stroke:none; font-size:28;">
@@ -534,6 +677,31 @@ class TestTextNode:
         assert main_group.contents[0].contents[3].y == -20 - (1.5 * 28)
 
 
+class TestRectNode:
+    def test_rx_ry(self):
+        converter = svglib.Svg2RlgShapeConverter(None)
+        node = svglib.NodeTracker(etree.XML(
+            '<rect rx="10" ry="4" width="10" height="3" x="0" y="0"/>'
+        ))
+        rect = converter.convertRect(node)
+        # rx/ry cannot be more than half the rect lengths.
+        assert rect.rx == 5
+        assert rect.ry == 1.5
+
+    def test_strokewidth_0(self):
+        """
+        When strokeWidth = 0, we force strokeColor to None to avoid the stroke
+        to be displayed.
+        """
+        converter = svglib.Svg2RlgShapeConverter(None)
+        node = svglib.NodeTracker(etree.XML(
+            '<rect style="stroke: rgb(100, 84, 0); fill: rgb(255, 255, 255); '
+            'fill-opacity: 1; stroke-width: 0;" width="135" height="86" x="10" y="10"/>'
+        ))
+        rect = converter.convertShape('rect', node)
+        assert rect.strokeColor is None
+
+
 class TestPolylineNode:
     def test_filling(self):
         converter = svglib.Svg2RlgShapeConverter(None)
@@ -600,7 +768,7 @@ class TestUseNode:
         ''')
 
     def test_transform_inherited_by_use(self):
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg version="1.1"
                  xmlns:xlink="http://www.w3.org/1999/xlink"
@@ -622,7 +790,7 @@ class TestUseNode:
         Sometimes, a node definition pointed to by xlink:href can appear after
         it has been referenced. But the order should remain.
         """
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg version="1.1"
                  xmlns:xlink="http://www.w3.org/1999/xlink"
@@ -639,7 +807,7 @@ class TestUseNode:
         """
         Properties on the use node apply to the referenced item.
         """
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg version="1.1"
                  xmlns:xlink="http://www.w3.org/1999/xlink"
@@ -663,7 +831,7 @@ class TestUseNode:
         different paths for filling and stroking), the use properties shouldn't
         affect the no-stroke property of the fake stroke-only path.
         """
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg xmlns="http://www.w3.org/2000/svg"
                  xmlns:xlink="http://www.w3.org/1999/xlink"
@@ -681,7 +849,7 @@ class TestUseNode:
 
 class TestSymbolNode:
     def test_symbol_node(self):
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg xmlns="http://www.w3.org/2000/svg"
                  xmlns:xlink="http://www.w3.org/1999/xlink"
@@ -701,7 +869,7 @@ class TestSymbolNode:
 
 class TestViewBox:
     def test_nonzero_origin(self):
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg xmlns="http://www.w3.org/2000/svg"
                  width="1200" height="800"
@@ -715,7 +883,7 @@ class TestViewBox:
         assert drawing.contents[0].transform == (10, 0, 0, -10, 600.0, 400.0)
 
     def test_percent_width_height(self):
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg xmlns="http://www.w3.org/2000/svg"
                  width="100%" height="100%"
@@ -728,7 +896,7 @@ class TestViewBox:
         assert (drawing.width, drawing.height) == (480, 360)
 
     def test_no_width_height(self):
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg xmlns="http://www.w3.org/2000/svg"
                  viewBox="0 0 480 360">
@@ -742,7 +910,7 @@ class TestViewBox:
 
 class TestEmbedded:
     def test_svg_in_svg(self):
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg xmlns="http://www.w3.org/2000/svg" version="1.1"
                  viewBox="0 0 210 297" height="297mm" width="210mm">
@@ -773,8 +941,21 @@ class TestEmbedded:
         assert pytest.approx(1.417, 0.001) == embedded_svg_group.getProperties()['transform'][0]
         assert pytest.approx(1.417, 0.001) == embedded_svg_group.getProperties()['transform'][3]
 
+    def test_embedded_svg_with_percentages(self):
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
+            <?xml version="1.0"?>
+            <svg xmlns="http://www.w3.org/2000/svg" version="1.1" height="100" width="100">
+                <rect height="100%" width="100%" x="0" y="0" />
+                <svg height="15" width="15" x="45" y="45">
+                    <rect fill="black" height="100%" width="100%" x="0" y="0" />
+                </svg>
+            </svg>
+        ''')))
+        inner_rect = drawing.contents[0].contents[1].contents[0]
+        assert inner_rect.width == 15
+
     def test_png_in_svg_file_like(self):
-        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
             <svg xmlns="http://www.w3.org/2000/svg" version="1.1"
                  xmlns:xlink="http://www.w3.org/1999/xlink"
